
/**
 * Which is better: Eclipse JDT null checking, or the checker framework?
 *
 * Many ways to compare: Which catches more errors. Which has fewer false
 * positives?
 *
 * Following example looks at a very ugly false positive
 *
 * http://wiki.eclipse.org/JDT_Core/Null_Analysis/Options#Lenient_analysis
 */


public class TestNullCode {

    @org.eclipse.jdt.annotation.Nullable
    private String testJdt;

    @org.checkerframework.checker.nullness.qual.Nullable
    private String testCheckerFramework;

    public void testJdt() {
        if (testJdt == null) {
        	testJdt = "not null";
        }
        // this line causes a false-positive warning
        // Potential null pointer access: The field testJdt is specified as @Nullable
        if (testJdt.equals("not null")) {
            System.out.println("will work because testJdt is not null");
        }

        if (testJdt != null) {
            // this used to be a bug but has been fixed:
            // http://stackoverflow.com/questions/27683817/eclipse-null-analysis-raises-false-positive-nullpointerexception-warning
            System.out.println(testJdt.toString());
        }
    }
    public void testJdtCopy() {
    	String copy = testJdt;
        if (copy == null) {
        	copy = "not null";
        }
        //This is fine
        if (copy.equals("not null")) {
            System.out.println("will work because testJdt is not null");
        }

        if (testJdt != null) {
            // this used to be a bug but has been fixed:
            // http://stackoverflow.com/questions/27683817/eclipse-null-analysis-raises-false-positive-nullpointerexception-warning
            System.out.println(testJdt.toString());
        }
    }

    public void testCheckerFramework() {
        if (testCheckerFramework == null) {
            testCheckerFramework = "not null";
        }
        // this line is ok
        if (testCheckerFramework.equals("not null")) {
            System.out.println("will work because testCheckerFramework is not null");
        }

        testCheckerFramework = null;
        // following line is flagged by checker framework
        // dereference of possibly-null reference testCheckerFramework
        System.out.println("will fail because testCheckerFramework is not null" + testCheckerFramework.toString());
    }
}
